@@ -1,10 +1,12 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 use clap::{Parser, Subcommand};
+use fs_extra::file::read_to_string;
 use gtk_rust_app::parse_project_descriptor;
 use nix::unistd::Uid;
 use std::{
-    fs::{create_dir_all, remove_dir_all},
+    fs::{create_dir_all, remove_dir_all, remove_file, File},
+    io::Write,
     path::{Path, PathBuf},
     process::Command,
 };
@@ -62,6 +64,12 @@ enum Gra {
 
 #[derive(Parser)]
 struct Flatpak {
+    #[clap(
+        short,
+        long,
+        help = "Create a release tar.xz of the sources and manifest.yml with according hash and the given url"
+    )]
+    release: Option<String>,
     #[clap(
         short,
         long,
@@ -103,7 +111,7 @@ fn main() {
                     .arg(format!(
                         "{}/{}.gschema.xml",
                         gra_gen.to_str().unwrap(),
-                        &descriptor.app.unwrap().id
+                        &descriptor.app.id
                     ))
                     .arg(GSETTINGS_SCHEMAS_DIR)
                     .output()
@@ -134,8 +142,7 @@ fn main() {
                 let o = Command::new("rm")
                     .arg(format!(
                         "{}/{}.gschema.xml",
-                        GSETTINGS_SCHEMAS_DIR,
-                        &descriptor.app.unwrap().id
+                        GSETTINGS_SCHEMAS_DIR, &descriptor.app.id
                     ))
                     .output()
                     .unwrap();
@@ -174,9 +181,120 @@ fn clean(target_dir: PathBuf) {
 
 fn flatpak(target_dir: PathBuf, arguments: Flatpak) {
     println!("[gra] Prepare flatpak build...");
-    let project_descriptor = parse_project_descriptor(Path::new("Cargo.toml")).unwrap();
+    let pd = parse_project_descriptor(Path::new("Cargo.toml"));
+    if pd.is_err() {
+        eprintln!("[gra] Could not parse Cargo.toml: {}", pd.unwrap_err());
+        return;
+    }
+    let project_descriptor = pd.unwrap();
     let gra_gen = target_dir.join("gra-gen");
 
+    if arguments.release.is_some() {
+        clean(target_dir.clone());
+        setup(target_dir.clone());
+
+        let flatpak_temp = match gtk_rust_app::prepare_flatpak_temp(&PathBuf::from(
+            target_dir.parent().unwrap(),
+        )) {
+            Ok(f) => f,
+            Err(e) => {
+                eprintln!("Could not prepare flatpak-temp: {:?}", e);
+                std::process::exit(-1);
+            }
+        };
+
+        let manifest_yml = target_dir.join(format!("{}.yml", &project_descriptor.app.id));
+
+        remove_file(gra_gen.join(format!("data/{}.dev.yml", &project_descriptor.app.id))).ok();
+        if let Err(e) = std::fs::rename(
+            gra_gen.join(format!("data/{}.yml", &project_descriptor.app.id)),
+            &manifest_yml,
+        ) {
+            eprintln!(
+                "[gra] Could not move target/gra-gen/data/{0}.yml to target/{0}.yml: {1}",
+                &project_descriptor.app.id,
+                e.to_string()
+            );
+            return;
+        }
+
+        let tar_file = target_dir.join(format!("{}.tar.xz", &project_descriptor.package.name));
+
+        println!("[gra] Call tar -C {:?} -cJf {:?} .", flatpak_temp, tar_file);
+        match Command::new("tar")
+            .args([
+                "-C",
+                flatpak_temp.to_str().unwrap(),
+                "-cJf",
+                tar_file.to_str().unwrap(),
+                ".",
+            ])
+            .spawn()
+        {
+            Err(e) => {
+                eprintln!(
+                    "[gra] tar flatpak-temp resulted in error: {}",
+                    e.to_string()
+                );
+                return;
+            }
+            Ok(mut c) => {
+                if let Err(e) = c.wait() {
+                    eprintln!("[gra] tar command failed: {}", e.to_string());
+                    return;
+                }
+            }
+        }
+        println!("[gra] Call sha256sum {:?}", &tar_file);
+
+        let sha = Command::new("sha256sum").arg(&tar_file).output();
+        if let Err(e) = sha {
+            eprintln!(
+                "[gra] sha256sum for {:?} resulted in error: {}",
+                &tar_file,
+                e.to_string()
+            );
+            return;
+        }
+
+        let sha = String::from_utf8_lossy(&sha.unwrap().stdout)
+            .to_string()
+            .split_once(" ")
+            .unwrap()
+            .0
+            .trim()
+            .to_string();
+
+        let manifest_template = read_to_string(&manifest_yml).unwrap();
+        let manifest = manifest_template
+            .replace("{archive}", &arguments.release.unwrap())
+            .replace("{sha256}", &sha);
+
+        let sha_file = target_dir.join(format!("{}.sha256.txt", &project_descriptor.package.name));
+
+        if let Ok(mut f) = File::create(&sha_file) {
+            f.write_all(sha.as_bytes()).unwrap();
+        } else {
+            eprintln!("[gra] Could not create {:?}", sha_file);
+            return;
+        }
+
+        if let Ok(mut f) = File::create(&manifest_yml) {
+            f.write_all(manifest.as_bytes()).unwrap();
+        } else {
+            eprintln!("[gra] Could not write to release {:?}", manifest_yml);
+            return;
+        }
+
+        println!(
+            "[gra] Created flatpak release file: {:?}, {}",
+            tar_file,
+            manifest_yml.to_string_lossy()
+        );
+
+        return;
+    }
+
     let flatpak_temp =
         match gtk_rust_app::prepare_flatpak_temp(&PathBuf::from(target_dir.parent().unwrap())) {
             Ok(f) => f,
@@ -193,10 +311,12 @@ fn flatpak(target_dir: PathBuf, arguments: Flatpak) {
     // setup flatpak-build dir
     let flatpak_build_rel = "../flatpak-build";
 
-    println!("[gra] Running flatpak-builder --repo={0}/repo {0}/host {1}/data/{2}.dev.yml --force-clean --state-dir={0}/state", 
+    println!("[gra] Running flatpak-builder --repo={0}/repo {0}/host {1}/data/{2}.dev.yml --force-clean --state-dir={0}/state {3}", 
         flatpak_build_rel,
         gra_gen.to_str().unwrap(),
-        &project_descriptor.app.as_ref().unwrap().id);
+        &project_descriptor.app.id,
+        arguments.arch.as_ref().map(|a| format!("--arch {}", a)).unwrap_or("".into())
+    );
 
     let mut c = Command::new("flatpak-builder");
     c.current_dir(&flatpak_temp);
@@ -211,10 +331,11 @@ fn flatpak(target_dir: PathBuf, arguments: Flatpak) {
     } else {
         c.arg(format!("{}/{}", flatpak_build_rel, "host"));
     }
-    c.arg(gra_gen.join("data").join(format!(
-        "{}.dev.yml",
-        project_descriptor.app.as_ref().unwrap().id
-    )));
+    c.arg(
+        gra_gen
+            .join("data")
+            .join(format!("{}.dev.yml", &project_descriptor.app.id)),
+    );
     let mut c = c.spawn().unwrap();
     c.wait().unwrap();
 
@@ -223,7 +344,7 @@ fn flatpak(target_dir: PathBuf, arguments: Flatpak) {
         flatpak_build_rel,
         target_dir.to_str().unwrap(),
         &project_descriptor.package.name,
-        &project_descriptor.app.as_ref().unwrap().id
+        &project_descriptor.app.id
     );
 
     let flatpak_file_name = format!("{}.flatpak", &project_descriptor.package.name);
@@ -233,7 +354,7 @@ fn flatpak(target_dir: PathBuf, arguments: Flatpak) {
         .arg("build-bundle")
         .arg(format!("{}/repo", flatpak_build_rel))
         .arg(format!("../{}", flatpak_file_name))
-        .arg(&project_descriptor.app.as_ref().unwrap().id)
+        .arg(&project_descriptor.app.id)
         .spawn()
         .unwrap();
     c.wait().unwrap();
